@@ -46,7 +46,7 @@ from __future__ import absolute_import
 from __future__ import print_function
 from __future__ import unicode_literals
 
-__version__ = "0.9"
+__version__ = "0.10"
 
 import math
 
@@ -75,7 +75,7 @@ class Collider(sge.Object):
        :attr:`y` manually will not cause any physics to occur.
     """
 
-    def move_x(self, move, absolute=False):
+    def move_x(self, move, absolute=False, do_events=True, exclude_events=()):
         """
         Move the object horizontally, handling physics.
 
@@ -87,7 +87,14 @@ class Collider(sge.Object):
           of vertical movement caused by slopes.  Otherwise, any
           vertical movement caused by slopes will result in a reduction
           of horizontal movement.
+        - ``do_events`` -- Whether or not physics collision events
+          should be executed when appropriate.
+        - ``exclude_events`` -- A set, list, or tuple of wall objects
+          which should not cause collision events to be executed if
+          collided with.
         """
+        exclude_events = set(exclude_events)
+        exclude_events.add(None)
         sticky = False
         move_mult = 1
         old_x = self.x
@@ -98,6 +105,36 @@ class Collider(sge.Object):
         old_bbox_bottom = self.bbox_bottom
         rold_bbox_top = round(old_bbox_top, NDIG)
         rold_bbox_bottom = round(old_bbox_bottom, NDIG)
+        on_floor = None
+        on_ceil = None
+
+        def get_on_floor(on_floor, self=self, old_x=old_x, old_y=old_y):
+            if on_floor is not None:
+                return on_floor
+            else:
+                new_x = self.x
+                new_y = self.y
+                self.x = old_x
+                self.y = old_y
+                on_floor = (self.get_bottom_touching_wall() or
+                            self.get_bottom_touching_slope())
+                self.x = new_x
+                self.y = new_y
+                return on_floor
+
+        def get_on_ceil(on_ceil, self=self, old_x=old_x, old_y=old_y):
+            if on_ceil is not None:
+                return on_ceil
+            else:
+                new_x = self.x
+                new_y = self.y
+                self.x = old_x
+                self.y = old_y
+                on_ceil = (self.get_top_touching_wall() or
+                           self.get_top_touching_slope())
+                self.x = new_x
+                self.y = new_y
+                return on_ceil
 
         if move > 0:
             bbb = round(self.bbox_bottom, NDIG)
@@ -135,7 +172,10 @@ class Collider(sge.Object):
 
             stopper = None
 
-            for other in self.collision(SlopeTopLeft):
+            slopes = self.collision(SlopeTopLeft)
+            def key(s, self=self): return s.get_slope_x(self.bbox_bottom)
+            slopes.sort(key=key)
+            for other in slopes:
                 y = other.get_slope_y(self.bbox_right)
                 if self.bbox_bottom > y:
                     oy = round(other.get_slope_y(old_bbox_right), NDIG)
@@ -147,15 +187,27 @@ class Collider(sge.Object):
                                 self.x -= move * (move_mult - m)
                                 move_mult = m
                                 y = other.get_slope_y(self.bbox_right)
-                        self.move_y(y - self.bbox_bottom)
+                        self.move_y(y - self.bbox_bottom, do_events=do_events,
+                                    exclude_events=exclude_events)
                         x = other.get_slope_x(self.bbox_bottom)
-                        self.bbox_right = min(self.bbox_right, x)
-                        stopper = other
+                        diff = self.bbox_right - x
+                        if diff > 0:
+                            self.bbox_right = x
+                            if self.bbox_bottom == y:
+                                self.move_x(diff, do_events=do_events,
+                                            exclude_events=exclude_events)
+
+                        on_floor = get_on_floor(on_floor)
+                        if on_floor:
+                            stopper = other
                     elif not self.collision(other, x=old_x):
                         self.bbox_right = min(self.bbox_right, other.bbox_left)
                         stopper = other
 
-            for other in self.collision(SlopeBottomLeft):
+            slopes = self.collision(SlopeBottomLeft)
+            def key(s, self=self): return s.get_slope_x(self.bbox_top)
+            slopes.sort(key=key)
+            for other in slopes:
                 y = other.get_slope_y(self.bbox_right)
                 if self.bbox_top < y:
                     oy = round(other.get_slope_y(old_bbox_right), NDIG)
@@ -167,10 +219,19 @@ class Collider(sge.Object):
                                 self.x -= move * (move_mult - m)
                                 move_mult = m
                                 y = other.get_slope_y(self.bbox_right)
-                        self.move_y(y - self.bbox_top)
+                        self.move_y(y - self.bbox_top, do_events=do_events,
+                                    exclude_events=exclude_events)
                         x = other.get_slope_x(self.bbox_top)
-                        self.bbox_right = min(self.bbox_right, x)
-                        stopper = other
+                        diff = self.bbox_right - x
+                        if diff > 0:
+                            self.bbox_right = x
+                            if self.bbox_top == y:
+                                self.move_x(diff, do_events=do_events,
+                                            exclude_events=exclude_events)
+
+                        on_ceil = get_on_ceil(on_ceil)
+                        if on_ceil:
+                            stopper = other
                     elif not self.collision(other, x=old_x):
                         self.bbox_right = min(self.bbox_right, other.bbox_left)
                         stopper = other
@@ -180,7 +241,7 @@ class Collider(sge.Object):
                     self.bbox_right = min(self.bbox_right, other.bbox_left)
                     stopper = other
 
-            if stopper is not None:
+            if do_events and stopper not in exclude_events:
                 move_loss = max(0, abs(move) - abs(self.x - old_x))
                 self.event_physics_collision_right(stopper, move_loss)
                 stopper.event_physics_collision_left(self, 0)
@@ -221,7 +282,10 @@ class Collider(sge.Object):
 
             stopper = None
 
-            for other in self.collision(SlopeTopRight):
+            slopes = self.collision(SlopeTopRight)
+            def key(s, self=self): return -s.get_slope_x(self.bbox_bottom)
+            slopes.sort(key=key)
+            for other in slopes:
                 y = other.get_slope_y(self.bbox_left)
                 if self.bbox_bottom > y:
                     oy = round(other.get_slope_y(old_bbox_left), NDIG)
@@ -233,15 +297,27 @@ class Collider(sge.Object):
                                 self.x -= move * (move_mult - m)
                                 move_mult = m
                                 y = other.get_slope_y(self.bbox_left)
-                        self.move_y(y - self.bbox_bottom)
+                        self.move_y(y - self.bbox_bottom, do_events=do_events,
+                                    exclude_events=exclude_events)
                         x = other.get_slope_x(self.bbox_bottom)
-                        self.bbox_left = max(self.bbox_left, x)
-                        stopper = other
+                        diff = self.bbox_left - x
+                        if diff < 0:
+                            self.bbox_left = x
+                            if self.bbox_bottom == y:
+                                self.move_x(diff, do_events=do_events,
+                                            exclude_events=exclude_events)
+
+                        on_floor = get_on_floor(on_floor)
+                        if on_floor:
+                            stopper = other
                     elif not self.collision(other, x=old_x):
                         self.bbox_left = max(self.bbox_left, other.bbox_right)
                         stopper = other
 
-            for other in self.collision(SlopeBottomRight):
+            slopes = self.collision(SlopeBottomRight)
+            def key(s, self=self): return -s.get_slope_x(self.bbox_top)
+            slopes.sort(key=key)
+            for other in slopes:
                 y = other.get_slope_y(self.bbox_left)
                 if self.bbox_top < y:
                     oy = round(other.get_slope_y(old_bbox_left), NDIG)
@@ -253,10 +329,19 @@ class Collider(sge.Object):
                                 self.x -= move * (move_mult - m)
                                 move_mult = m
                                 y = other.get_slope_y(self.bbox_left)
-                        self.move_y(y - self.bbox_top)
+                        self.move_y(y - self.bbox_top, do_events=do_events,
+                                    exclude_events=exclude_events)
                         x = other.get_slope_x(self.bbox_top)
-                        self.bbox_left = max(self.bbox_left, x)
-                        stopper = other
+                        diff = self.bbox_left - x
+                        if diff < 0:
+                            self.bbox_left = x
+                            if self.bbox_top == y:
+                                self.move_x(diff, do_events=do_events,
+                                            exclude_events=exclude_events)
+
+                        on_ceil = get_on_ceil(on_ceil)
+                        if on_ceil:
+                            stopper = other
                     elif not self.collision(other, x=old_x):
                         self.bbox_left = max(self.bbox_left, other.bbox_right)
                         stopper = other
@@ -266,7 +351,7 @@ class Collider(sge.Object):
                     self.bbox_left = max(self.bbox_left, other.bbox_right)
                     stopper = other
 
-            if stopper is not None:
+            if do_events and stopper not in exclude_events:
                 move_loss = max(0, abs(move) - abs(self.x - old_x))
                 self.event_physics_collision_left(stopper, move_loss)
                 stopper.event_physics_collision_right(self, 0)
@@ -334,7 +419,7 @@ class Collider(sge.Object):
                 if new_bbox_top is not None:
                     self.bbox_top = new_bbox_top
 
-    def move_y(self, move, absolute=False):
+    def move_y(self, move, absolute=False, do_events=True, exclude_events=()):
         """
         Move the object vertically, handling physics.
 
@@ -346,7 +431,14 @@ class Collider(sge.Object):
           of horizontal movement caused by slopes.  Otherwise, any
           horizontal movement caused by slopes will result in a
           reduction of vertical movement.
+        - ``do_events`` -- Whether or not physics collision events
+          should be executed when appropriate.
+        - ``exclude_events`` -- A set, list, or tuple of wall objects
+          which should not cause collision events to be executed if
+          collided with.
         """
+        exclude_events = set(exclude_events)
+        exclude_events.add(None)
         sticky = False
         move_mult = 1
         old_x = self.x
@@ -357,6 +449,36 @@ class Collider(sge.Object):
         rold_bbox_right = round(old_bbox_right, NDIG)
         old_bbox_top = self.bbox_top
         old_bbox_bottom = self.bbox_bottom
+        on_right = None
+        on_left = None
+
+        def get_on_right(on_right, self=self, old_x=old_x, old_y=old_y):
+            if on_right is not None:
+                return on_right
+            else:
+                new_x = self.x
+                new_y = self.y
+                self.x = old_x
+                self.y = old_y
+                on_right = (self.get_right_touching_wall() or
+                            self.get_right_touching_slope())
+                self.x = new_x
+                self.y = new_y
+                return on_right
+
+        def get_on_left(on_left, self=self, old_x=old_x, old_y=old_y):
+            if on_left is not None:
+                return on_left
+            else:
+                new_x = self.x
+                new_y = self.y
+                self.x = old_x
+                self.y = old_y
+                on_left = (self.get_left_touching_wall() or
+                           self.get_left_touching_slope())
+                self.x = new_x
+                self.y = new_y
+                return on_left
 
         if move > 0:
             bbr = round(self.bbox_right, NDIG)
@@ -394,7 +516,10 @@ class Collider(sge.Object):
 
             stopper = None
 
-            for other in self.collision(SlopeTopLeft):
+            slopes = self.collision(SlopeTopLeft)
+            def key(s, self=self): return s.get_slope_y(self.bbox_right)
+            slopes.sort(key=key)
+            for other in slopes:
                 x = other.get_slope_x(self.bbox_bottom)
                 if self.bbox_right > x:
                     ox = round(other.get_slope_x(old_bbox_bottom), NDIG)
@@ -406,16 +531,28 @@ class Collider(sge.Object):
                                 self.y -= move * (move_mult - m)
                                 move_mult = m
                                 x = other.get_slope_x(self.bbox_bottom)
-                        self.move_x(x - self.bbox_right)
+                        self.move_x(x - self.bbox_right, do_events=do_events,
+                                    exclude_events=exclude_events)
                         y = other.get_slope_y(self.bbox_right)
-                        self.bbox_bottom = min(self.bbox_bottom, y)
-                        stopper = other
+                        diff = self.bbox_bottom - y
+                        if diff > 0:
+                            self.bbox_bottom = y
+                            if self.bbox_right == x:
+                                self.move_y(diff, do_events=do_events,
+                                            exclude_events=exclude_events)
+
+                        on_right = get_on_right(on_right)
+                        if on_right:
+                            stopper = other
                     elif not self.collision(other, y=old_y):
                         self.bbox_bottom = min(self.bbox_bottom,
                                                other.bbox_top)
                         stopper = other
 
-            for other in self.collision(SlopeTopRight):
+            slopes = self.collision(SlopeTopRight)
+            def key(s, self=self): return s.get_slope_y(self.bbox_left)
+            slopes.sort(key=key)
+            for other in slopes:
                 x = other.get_slope_x(self.bbox_bottom)
                 if self.bbox_left < x:
                     ox = round(other.get_slope_x(old_bbox_bottom), NDIG)
@@ -427,10 +564,19 @@ class Collider(sge.Object):
                                 self.y -= move * (move_mult - m)
                                 move_mult = m
                                 x = other.get_slope_x(self.bbox_bottom)
-                        self.move_x(x - self.bbox_left)
+                        self.move_x(x - self.bbox_left, do_events=do_events,
+                                    exclude_events=exclude_events)
                         y = other.get_slope_y(self.bbox_left)
-                        self.bbox_bottom = min(self.bbox_bottom, y)
-                        stopper = other
+                        diff = self.bbox_bottom - y
+                        if diff > 0:
+                            self.bbox_bottom = y
+                            if self.bbox_left == x:
+                                self.move_y(diff, do_events=do_events,
+                                            exclude_events=exclude_events)
+
+                        on_left = get_on_left(on_left)
+                        if on_left:
+                            stopper = other
                     elif not self.collision(other, y=old_y):
                         self.bbox_bottom = min(self.bbox_bottom,
                                                other.bbox_top)
@@ -441,7 +587,7 @@ class Collider(sge.Object):
                     self.bbox_bottom = min(self.bbox_bottom, other.bbox_top)
                     stopper = other
 
-            if stopper is not None:
+            if do_events and stopper not in exclude_events:
                 move_loss = max(0, abs(move) - abs(self.y - old_y))
                 self.event_physics_collision_bottom(stopper, move_loss)
                 stopper.event_physics_collision_top(self, 0)
@@ -482,7 +628,10 @@ class Collider(sge.Object):
 
             stopper = None
 
-            for other in self.collision(SlopeBottomLeft):
+            slopes = self.collision(SlopeBottomLeft)
+            def key(s, self=self): return -s.get_slope_y(self.bbox_right)
+            slopes.sort(key=key)
+            for other in slopes:
                 x = other.get_slope_x(self.bbox_top)
                 if self.bbox_right > x:
                     ox = round(other.get_slope_x(old_bbox_top), NDIG)
@@ -494,14 +643,26 @@ class Collider(sge.Object):
                                 self.y -= move * (move_mult - m)
                                 move_mult = m
                                 x = other.get_slope_x(self.bbox_top)
-                        self.move_x(x - self.bbox_right)
+                        self.move_x(x - self.bbox_right, do_events=do_events,
+                                    exclude_events=exclude_events)
                         y = other.get_slope_y(self.bbox_right)
-                        self.bbox_top = max(self.bbox_top, y)
-                        stopper = other
+                        diff = self.bbox_top - y
+                        if diff < 0:
+                            self.bbox_top = y
+                            if self.bbox_right == x:
+                                self.move_y(diff, do_events=do_events,
+                                            exclude_events=exclude_events)
+
+                        on_right = get_on_right(on_right)
+                        if on_right:
+                            stopper = other
                     elif not self.collision(other, y=old_y):
                         self.bbox_top = max(self.bbox_top, other.bbox_bottom)
                         stopper = other
 
+            slopes = self.collision(SlopeBottomRight)
+            def key(s, self=self): return -s.get_slope_y(self.bbox_left)
+            slopes.sort(key=key)
             for other in self.collision(SlopeBottomRight):
                 x = other.get_slope_x(self.bbox_top)
                 if self.bbox_left < x:
@@ -514,10 +675,19 @@ class Collider(sge.Object):
                                 self.y -= move * (move_mult - m)
                                 move_mult = m
                                 x = other.get_slope_x(self.bbox_top)
-                        self.move_x(x - self.bbox_left)
+                        self.move_x(x - self.bbox_left, do_events=do_events,
+                                    exclude_events=exclude_events)
                         y = other.get_slope_y(self.bbox_left)
-                        self.bbox_top = max(self.bbox_top, y)
-                        stopper = other
+                        diff = self.bbox_top - y
+                        if diff < 0:
+                            self.bbox_top = y
+                            if self.bbox_left == x:
+                                self.move_y(diff, do_events=do_events,
+                                            exclude_events=exclude_events)
+
+                        on_left = get_on_left(on_left)
+                        if on_left:
+                            stopper = other
                     elif not self.collision(other, y=old_y):
                         self.bbox_top = max(self.bbox_top, other.bbox_bottom)
                         stopper = other
@@ -527,7 +697,7 @@ class Collider(sge.Object):
                     self.bbox_top = max(self.bbox_top, other.bbox_bottom)
                     stopper = other
 
-            if stopper is not None:
+            if do_events and stopper not in exclude_events:
                 move_loss = max(0, abs(move) - abs(self.y - old_y))
                 self.event_physics_collision_top(stopper, move_loss)
                 stopper.event_physics_collision_bottom(self, 0)
@@ -658,7 +828,7 @@ class Collider(sge.Object):
         for slope in self.collision(SlopeBottomRight, x=(self.x - 1)):
             y = round(slope.get_slope_y(self.bbox_left), NDIG)
             if bbt == y or (self.bbox_top <= slope.bbox_top and
-                            not self.collision(sope)):
+                            not self.collision(slope)):
                 r.append(slope)
 
         return r
@@ -895,7 +1065,8 @@ class SlopeTopLeft(sge.Object):
         # x = (y - b) / m [b is 0]
         m = -self.bbox_height / self.bbox_width
         y -= self.bbox_top
-        return y / m + self.bbox_right
+        x = y / m + self.bbox_right
+        return max(self.bbox_left, min(x, self.bbox_right))
 
     def get_slope_y(self, x):
         """
@@ -905,7 +1076,8 @@ class SlopeTopLeft(sge.Object):
         # y = mx + b [b is 0]
         m = -self.bbox_height / self.bbox_width
         x -= self.bbox_left
-        return m * x + self.bbox_bottom
+        y = m * x + self.bbox_bottom
+        return max(self.bbox_top, min(y, self.bbox_bottom))
 
     def event_physics_collision_left(self, other, move_loss):
         """
@@ -959,7 +1131,8 @@ class SlopeTopRight(sge.Object):
         # x = (y - b) / m [b is 0]
         m = self.bbox_height / self.bbox_width
         y -= self.bbox_top
-        return y / m + self.bbox_left
+        x = y / m + self.bbox_left
+        return max(self.bbox_left, min(x, self.bbox_right))
 
     def get_slope_y(self, x):
         """
@@ -969,7 +1142,8 @@ class SlopeTopRight(sge.Object):
         # y = mx + b [b is 0]
         m = self.bbox_height / self.bbox_width
         x -= self.bbox_left
-        return m * x + self.bbox_top
+        y = m * x + self.bbox_top
+        return max(self.bbox_top, min(y, self.bbox_bottom))
 
     def event_physics_collision_right(self, other, move_loss):
         """
@@ -1023,7 +1197,8 @@ class SlopeBottomLeft(sge.Object):
         # x = (y - b) / m [b is 0]
         m = self.bbox_height / self.bbox_width
         y -= self.bbox_top
-        return y / m + self.bbox_left
+        x = y / m + self.bbox_left
+        return max(self.bbox_left, min(x, self.bbox_right))
 
     def get_slope_y(self, x):
         """
@@ -1033,7 +1208,8 @@ class SlopeBottomLeft(sge.Object):
         # y = mx + b [b is 0]
         m = self.bbox_height / self.bbox_width
         x -= self.bbox_left
-        return m * x + self.bbox_top
+        y = m * x + self.bbox_top
+        return max(self.bbox_top, min(y, self.bbox_bottom))
 
     def event_physics_collision_left(self, other, move_loss):
         """
@@ -1087,7 +1263,8 @@ class SlopeBottomRight(sge.Object):
         # x = (y - b) / m [b is 0]
         m = -self.bbox_height / self.bbox_width
         y -= self.bbox_top
-        return y / m + self.bbox_right
+        x = y / m + self.bbox_right
+        return max(self.bbox_left, min(x, self.bbox_right))
 
     def get_slope_y(self, x):
         """
@@ -1097,7 +1274,8 @@ class SlopeBottomRight(sge.Object):
         # y = mx + b [b is 0]
         m = -self.bbox_height / self.bbox_width
         x -= self.bbox_left
-        return m * x + self.bbox_bottom
+        y = m * x + self.bbox_bottom
+        return max(self.bbox_top, min(y, self.bbox_bottom))
 
     def event_physics_collision_right(self, other, move_loss):
         """
@@ -1449,7 +1627,7 @@ class MobileColliderWall(MobileWall, Collider):
        should be pushing.
     """
 
-    def move_x(self, move, absolute=False):
+    def move_x(self, move, absolute=False, do_events=True, exclude_events=()):
         """
         Move the wall horizontally, handling physics.
 
@@ -1458,12 +1636,12 @@ class MobileColliderWall(MobileWall, Collider):
         - ``move`` -- The amount to add to :attr:`y`.
         """
         xprev = self.x
-        Collider.move_x(self, move, absolute)
+        Collider.move_x(self, move, absolute, do_events, exclude_events)
         real_move = self.x - xprev
         self.x = xprev
         MobileWall.move_x(self, real_move)
 
-    def move_y(self, move, absolute=False):
+    def move_y(self, move, absolute=False, do_events=True, exclude_events=()):
         """
         Move the wall vertically, handling physics.
 
@@ -1472,7 +1650,7 @@ class MobileColliderWall(MobileWall, Collider):
         - ``move`` -- The amount to add to :attr:`y`.
         """
         yprev = self.y
-        Collider.move_y(self, move, absolute)
+        Collider.move_y(self, move, absolute, do_events, exclude_events)
         real_move = self.y - yprev
         self.y = yprev
         MobileWall.move_y(self, real_move)
